/*!
\file Point3.cpp
\brief encapsulates a 3D point
\author Jonathan Macey
\version 1.0
\date 23/11/05
\warning Real x,y,z must be first data elements in the class as these are accessed by the [] operator
*/

#include "Point3.h"
#include "myassert.h"
using namespace GraphicsLib;
bool Point3 :: operator==(const Point3& rhs)
{
return (x==rhs.x && y==rhs.y && z==rhs.z);	
}		
	
bool Point3 :: operator!=(const Point3& rhs)
{
return (x!=rhs.x || y!=rhs.y || z!=rhs.z);
}


Point3 Point3 :: operator+(const Point3& rhs)
{
return Point3(x+rhs.x,y+rhs.y,z+rhs.z); 
}

Point3 Point3 :: operator+(const Point3* rhs)
{

return Point3(x+rhs->x,y+rhs->y,z+rhs->z); 
}
  
void Point3::operator+=(const Point3 &rhs)
{
x+=rhs.x; y+=rhs.y; z+=rhs.z; 
}

void Point3::operator+=(const Vector &rhs)
{
x+=rhs.x; y+=rhs.y; z+=rhs.z;
}

void Point3::operator-=(const Point3 &rhs)
{
x-=rhs.x; y-=rhs.y; z-=rhs.z;
}
void Point3::operator-=(const Vector &rhs)
{
x-=rhs.x; y-=rhs.y; z-=rhs.z;
}

Vector Point3 :: operator-(const Point3& rhs)
{
return Vector(x-rhs.x,y-rhs.y,z-rhs.z,0.0);
}


Point3 Point3 :: operator-(const Vector& rhs)
{
return Point3(x-rhs.x,y-rhs.y,z-rhs.z);
}

Point3 Point3 :: operator+(const Vector& rhs)
{
return Point3(x+rhs.x,y+rhs.y,z+rhs.z); 
}

Point3 Point3::operator*(Real rhs)
{
return Point3 (x*rhs,y*rhs,z*rhs);
}

void Point3::operator*=(Real rhs)
{
x*=rhs; y*=rhs; z*=rhs; 
}

Point3 Point3::operator*(const Vector &rhs)
{
return Point3(x*rhs.x,y*rhs.y, z*rhs.z);
}

Point3 Point3::operator*(const Point3 &rhs)
{
return Point3 (x*rhs.x, y*rhs.y,z*rhs.z);

}


Point3 Point3::operator/(Real rhs)
{
assert(rhs !=0);
return Point3(x/rhs,y/rhs,z/rhs); 
}

Real Point3::operator[](int i)
{
// make sure we only access the x,y,z components
assert(i >=0 || i<=3);
return (&x)[i]; 	
}

Point3 Point3::operator-()
{ 
return Point3(-x,-y,-z); 
}

Point3& Point3::operator=(const Point3 &rhs)
{
x=rhs.x; y=rhs.y; z=rhs.z;
return *this;	
}

void Point3 :: Vertex(void)
{
glVertex3f(x,y,z);
}

void Point3 :: Translate(void)
{
glTranslatef(x,y,z);
}

void Point3 :: Normal(void)
{
glNormal3f(x,y,z);
}


std::ostream& GraphicsLib::operator<<(std::ostream& output, const GraphicsLib::Point3& s)
 {
    return output<<"["<<s.x<<","<<s.y<<","<<s.z<<"]";
 }
std::ostream& GraphicsLib::operator<<(std::ostream& output, const GraphicsLib::Point3* s)
 {
    return output<<"["<<s->x<<","<<s->y<<","<<s->z<<"]";
 }

std::istream& GraphicsLib::operator>>(std::istream& input, GraphicsLib::Point3 &s)
{
return input >> s.x >> s.y >> s.z;

}

std::istream& GraphicsLib::operator>>(std::istream& input, GraphicsLib::Point3 *s)
{
return input >> s->x >> s->y >> s->z;

}

void Point3::normalize(void)
{
Real len=(Real)sqrt(x*x+y*y+z*z);
assert(len!=0);
x/=len; y/=len; z/=len;
} 


    
Point3 Point3::operator*(const Matrix &rhs)
{
  Point3 a;
  Real *f=(Real *)&rhs;
  a.x = x*f[0] + y*f[4] + z*f[8] +f[3];
  a.y = x*f[1] + y*f[5] + z*f[9] +f[7]; 
  a.z = x*f[2] + y*f[6] + z*f[10] +f[11];
  return a;
}


    
    
