#ifndef __MATRIX_H_
#define __MATRIX_H_

#include <string.h> // for memset
#include "Vector.h"
#include <GL/gl.h>
#include <iostream>
#include <math.h>
#include "types.h"
/*!Matrix Class to do simple matrix operations included 
 * operator overloaded functions for maths 
 * and matrix * vector type maths */
namespace GraphicsLib {
class Matrix
{
 friend std::ostream& operator<<(std::ostream& output, const Matrix& M);
public:
	/*! matrix element m as a 4x4 array */
	Real m[4][4];
	Matrix();
    Matrix(const Matrix& M);
    Matrix(
    		Real m00,Real m01,Real m02, Real m03,
    		Real m10,Real m11,Real m12, Real m13,
            Real m20,Real m21,Real m22, Real m23,
            Real m30,Real m31,Real m32, Real m33
         );
	/*! ToReal will convert the matrix to an all Real
	 * value
	 * \param Real in[16] the value to save to
	 */

	void ToReal(Real in[16]);
	/*!set the value at M[x][y] to equals
	 * \param GLint x the x index into the array
	 * \param GLint y the y index into the array
	 * \param Real equals the value to set to
	 */
    void Mi(GLint x,GLint y,Real equals);
	
	// clears matrix to all 0
	/*! clear the matrix to all 0 */
	void null(void);
	/*! make the matrix m the identity matrix <BR>
	 * 1 0 0 0 <BR>
	 * 0 1 0 0 <BR>
	 * 0 0 1 0 <BR>
	 * 0 0 0 1 <BR>
	 */
	// set as identity matrix
	void Identity(void);
	

Matrix operator*(const Matrix &m1);
void operator*= (const Matrix &m1);
Matrix operator+(const Matrix &m1);
void operator+=(const Matrix &m1);
Matrix operator*(const Real i);
void operator*=(const Real i);
void Transpose(void);
void LoadModelView(void);
void LoadProjection(void);
void RotateX(Real deg);
void RotateY(Real deg);
void RotateZ(Real deg);
void Translate(Real x, Real y, Real z);
void Scale(Real x,Real y, Real z);
double Determinant(void);

};
}

#endif
