#ifndef __Camera_H__
#define __Camera_H__
#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif
#include "types.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include "Point3.h"
#include "Vector.h"
#include "Matrix.h"
#include <math.h>
// a lot of this stuff is from the HILL book Computer Graphics with OpenGL 2nd Ed Prentice Hall
// a very good book

namespace GraphicsLib {
class Camera{
private:
	
public:
	/*! << extraction operator to prints out the colour in the format [r,g,b,a]
   \param std::ostream & output the outstream class pointer
   \param const Colour &s the colour to print out 
*/
friend std::ostream& operator<<(std::ostream& output, const Camera& c);
	void setModelViewMatrix();
    Vector u, v, n;
	Point3 Eye;
	Real aspect, nearDist, farDist, viewAngle;
	Point3 Look;
	Vector UP;
	Real W,H;
	GLint nRows, nCols;
    Real rollAngle;
    Real pitchAngle;
    Real yawAngle;
	Camera()
	{ // make default camera
	   setDefaultCamera();
	}
    Camera(Point3 eye, Point3 look, Vector Up);
	void printCamera(void);
	void roll(Real angle);
	void pitch(Real angle);
	void yaw(Real angle);
	void slide(GLdouble du, GLdouble dv, GLdouble dn);
	void set(Point3 Eye,  Point3 look, Vector up);
	void set(Real ex, Real ey, Real ez, Real lx, Real ly, Real lz, Real upx, Real upy, Real upz);
	void setDisplay(GLint numRows, GLint numCols);
	void setProjectionMatrix();
	void setShape(Real viewAngle, Real aspect, Real Near, Real Far);
	void setAspect(Real asp);
	void getShape(Real& viewAngle, Real& aspect, Real& Near, Real& Far);
	void rotAxes(Vector& a, Vector& b, Real angle);
	void setDefaultCamera();
    // moves the camera along the world coordinate system by dx, dy, dz
    void move(double dx, double dy, double dz);
	void moveBoth(double dx,double dy,double dz);
    void moveEye(double dx, double dy, double dz);
	void moveLook(double dx, double dy, double dz);
    void setViewAngle(Real Angle);
	void Print(void);
    void Use(void);
    void normalisedYaw(Real angle);
};
}
#endif // end of  class

