//---------------------------------------------------------------------------

#ifndef CameraControllerH
#define CameraControllerH
#include "types.h"
#include <math.h>
#include "Camera.h" //access to camera, point, and vector
//---------------------------------------------------------------------------

// This structure holds the information about a control event
namespace GraphicsLib {
struct CameraControlEvent
{
    int EventTime; // ms since sequence start by which time these values will be met

    Real Pitch;    // in degrees
    Real Yaw;      // in degrees
    Real Roll;     // in degrees

    // camera position
    Real X;
    Real Y;
    Real Z;

    // for linked list of events
    CameraControlEvent *Next;

    // simple constructor to null next pointer
    CameraControlEvent() {Next = NULL; };

};

//---------------------------------------------------------------------------
class CameraController
{
public:
    CameraController(Camera *cam);
    ~CameraController();

    void LoadControlScript(char *filename); // loads a .cam file

    // should be called by the render loop to allow the controller
    // to update the camera
    // Inputs: the time elapsed in MS for this update
    void Update(int elapsed);

    // starts the sequence
    void Start();
    // stops (pauses) the sequence
    void Stop();
    // resets the sequence
    void Reset();

private:
    // unloads the events and frees up memory
    void Clear();

    // class states
    bool Loaded;
    bool Running;

    // current time (ms) elapsed since sequence started
    int SequenceTime;

    // linked list of events
    CameraControlEvent *Events;

    // pointer to the current event
    CameraControlEvent *CurrentEvent;

    // values for the current event
    Real Pitch;
    Real Yaw;
    Real Roll;
    Real X;
    Real Y;
    Real Z;

    // pointer to camere
    Camera *Cam;

};
}

#endif
