#ifndef __COLOUR_H__
#define __COLOUR_H__



#ifdef _WINDOWS_
#include <windows.h>
#endif
#include "types.h"
#include <GL/gl.h>
#include <math.h>
#include <iostream>
namespace GraphicsLib {
class Colour
{ // holds an red,green,blue 3-tuple
public :
	/*!Real red component of the colour tuple */
	Real r;
	/*!Real green component of the colour tuple */
	Real g;
	/*!Real blue component of the colour tuple */
	Real b;
	/*!Real  alpha the transparent element */
	Real a;

	/*!default constructor set all values to 0 */
	  inline Colour(){r = g = b =  a= 0.0;}
	/*! constructor passing in r g b components
	 * \param Real R red component
	 * \param Real G green component
	 * \param Real B blue component */
	  inline Colour(Real R, Real G, Real B)
		{r = R; g = G; b = B;}
	  /*! constructor passing in r g b components
	 * \param Real R red component
	 * \param Real G green component
	 * \param Real B blue component
     * \param Real A the alpha component
     */
     
	  inline Colour(Real R, Real G, Real B, Real A)
		{r = R; g = G; b = B;a=A;}

	/*! constructor passing in a Colour
	 * \Colour the colour to use
	 */
	  inline Colour(Colour const &c)
		{r = c.r; g = c.g; b = c.b; a=c.a;}

	/*! set method to change the current values of the colour
	 * \param Real r red component to be set
	 * \param Real g green component to be set
	 * \param Real b blue component to be set
	 */
	inline void set(Real R, Real G, Real B)
		{r = R; g = G; b = B; a=1.0;}
	/*! set method to change the current values of the colour
	 * as well as the transparency
	 * \param Real r red component to be set
	 * \param Real g green component to be set
	 * \param Real b blue component to be set
	 * \param Real alpha transparency value
	 */
	inline void set(Real R, Real G,Real B, Real A)
		{r=R; g=G; b=B; a=A;}
	/*! constructor passing in a Colour
	 * \Colour the colour to use
	 */
	inline void set(Colour c)
      {r = c.r; g = c.g; b= c.b; a=c.a;}

	Colour operator +(const Colour &rhs);
	Colour operator -(const Colour &rhs);
    void operator +=(const Colour &rhs);
    Colour operator *(const Colour &rhs);
    Colour operator *(Real rhs);
    void operator *=(const Colour &rhs);
    void operator *=(Real rhs);
	inline void add(Colour src, Colour refl);
	void build4tuple(Real v[]);
/*! << extraction operator to prints out the colour in the format [r,g,b,a]
   \param std::ostream & output the outstream class pointer
   \param const Colour &s the colour to print out
*/
friend std::ostream& operator<<(std::ostream& output, const Colour& s);
/*! >> insertion operator to read in the colour data r -> g -> b -> a
   \param std::istream & input the instream class pointer
   \param const Colour s the colour to read in to
*/
friend std::istream& operator>>(std::istream& input, Colour s);
/*! call glColor4f to set this as the current colour */
void Use(void);
};

}

#endif // colour
