#ifndef __CURVEGEN_H_
#define __CURVEGEN_H_

#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif
#include "types.h"
#include <GL/glut.h>
#include "Point3.h"
#include "Util.h"
#include <math.h>
/*! \class CurveFollow.h <BR>
	Author Jonathan Macey <BR>
    Version 3.0 <BR>
    Date Last Revision 31/01/03 <BR>
    Linkages : AgentEmitter.h <BR>
	PreProcessor : If using windows define __USINGWINDOWS__ flag
                   For debug version define __DEBUG_ so it doesn't conflict with the windows one <BR>
    Description : <BR>
    Simple 4 point spline for Agents to follow, each emitter has 1 spline for the agents to follow
    may be updated to n point spline in later versions uses the deCastejau algorith and the
    tween3 function from Util.h <BR>
*/
namespace GraphicsLib {
class CurveFollow
{
public :
  /*! Point3 CP[4] array of control points for Curve */
  Point3 CP[4];
  /*! Constructor for CurveFollow passes in the 4 control points
   \param Point3 p[4] array of points for curve */
  CurveFollow(Point3 p[4]);
  /*! destructor */
  ~CurveFollow();
  /*! SetCP sets the control point at index to the point passed in
  \param int index the index into the control point array
  \param Point3 P the point passed in to set */
  void SetCP(int index, Point3 P);
  /*! GetPointOnCurve returns the point on the curve at time t where t = 0..1
  \param Real t the point on the curve
  \returns Point3 the point at time t */
  Point3 GetPointOnCurve(Real t);
  /*! DrawCurve draws the curve */
  void DrawCurve(void);

  private :


};

}



#endif
