#ifndef __LIGHT_H__
#define __LIGHT_H__

#include "Point3.h"
#include "Colour.h"
#include <GL/gl.h>
#include <math.h>
#include "types.h"
namespace GraphicsLib {
/*! \enum LIGHTMODES 
 *  used to flag if a light is local or remote to the scene
 *
 *   LIGHTREMOTE used to set the light remote <BR>
 *   LIGHTLOCAL used to set the light local
 */  
enum LIGHTMODES {LIGHTREMOTE,LIGHTLOCAL} ; //#define LIGHTREMOTE 0
//#define LIGHTLOCAL 1
class Light
{ // for a linked list of light source color and position
protected :
	/*! Point3 pos used to store the light position */ 	
    Point3 pos;
    /*! Colour colour used to give the light a colour */
	Colour colour;
	Colour SpecColour;
    /*! int LightNo used to set the light number this is a direct map to
	 *  OpenGL light number GL_LIGHT[x] where x is a number. The internal
	 *  implimentation will use GL_LIGHT0+LightNo to indicate light numbers */
	GLint LightNo;
	/*!used to hold the current lights mode */
	LIGHTMODES LightMode;
	/*! Allows us to create a linked list of lights */
public:
	inline void setPosition(Point3 p){pos.set(p);}
	inline void setColour(Colour c){colour.set(c);}
    inline void setSpecColour(Colour c){SpecColour.set(c);}
	/*! Simple constructor which sets no parameters apart from light
	 *  Number so we can use the OpenGL default light values
	 *  Which are usually quite good
	 */
	inline Light(GLint number){LightNo=number;}
	/*! default constructor */
	inline Light(){;}
	Light(GLint number, Point3 Pos, Colour col,LIGHTMODES lightmode);
    Light(GLint number, Point3 Pos, Colour col,Colour specol,LIGHTMODES lightmode);
	/*! destructor when light is destroyed we turn it off 
	 *  by using glDisable(GL_LIGHT0+LightNo); */
	inline ~Light() { glDisable(GL_LIGHT0+LightNo);}
	/*! Disable the light by calling glDisable(GL_LIGHT0+LightNo); */
	inline void Disable(){glDisable(GL_LIGHT0+LightNo); }
	void Enable();
	/*! return the next light if using a linked list of lights */
	
    void Show(void);		
/*! returns the current value of the light number
 *  \returns int LightNo */
	GLint getLightNum(){return LightNo;}
/*! returns the current light position as a Point3
 * \returns Point3 pos */
	Point3 getPos(void){return pos;}
/*! returns the current light colour
 * \returns Colour coloour */
	Colour getColour(void){return colour;}
	
};

}


#endif // __LIGHT__
