#ifndef __UTIL_H__
#define __UTIL_H__
#include "Point3.h"
#include <math.h>
#ifdef _WINDOWS_
	#include <windows.h>
#endif
#include <GL/gl.h>
#include "types.h"
namespace GraphicsLib
{
#define TWO_PI 6.28318530717958647692   //360
#define PI 3.14159265358979323846       //180
#define PI2 1.57079632679489661923      //90
#define PI4 0.785398163397448309615     //45

// /*! Tween3 used to find the point t along two points
//  * \param Point3 A the first point
//  * \param Point3 B the second point
//  * \param Real the fraction along the line
//  * \return Point3 the new point
//  */
// extern Point3 Tween3(Point3 A,Point3 B, Real t);
// 
// /*! lerp simple linear interpolation funciton for 2 Reals
//    \param Real A the first value
//  * \param Real B the second value
//  * \param Real the fraction along the line
//  * \return Real the new value
//  */
// extern Real lerp(Real A,Real B, Real t);

/*!DrawSquare draws a square centered at x,y of size/2
 * \param Real x the x position
 * \param Real y the y position
 * \param Real size the size of the square
 */
extern void DrawSquare(Real x, Real y,Real size);

/*!DrawAxis draws a 3d Axis at the current position
 * Weere the axis are represented as follows <BR>
X=RED
Y=GREEN
Z=BLUE
* \param Real Size the scale factor for the axis
*/
extern void DrawAxis(Real Size);


/*calculates the normal from 3 points and return the new
 * normal as a Vector
 * \param Point3 p1 the first point
 * \param Point3 p2 the second point
 * \param Point3 p4 the third point
 * \return Vector the normal of the 3 points
 */


extern Vector CalcNormal( Point3 p1, Point3 p2, Point3 p3 );
/*!converts Degrees to Radians
 * \param Real deg the angle to convert
 * \param the angle in Radians
 */
extern Real DegtoRad(Real deg);
/*! converts Radians to Degrees
 * \param Real rad the angle in radians
 * \returns the angle in Degrees
 */
extern Real RadtoDeg(Real rad);

extern void DrawAxisSimple(Real Size);
extern Real RandomNum();
extern Real RandomNum(Real MaxVal);
extern Real RandomPosNum(Real MaxVal);
extern void SetSeed(void);
/*! Lerp function to to blend a with b by ammount t
\param T a value 1
\param T b value 2
\param Real t the blend ammount
\returns <class T> blended value
*/
template <class T> T Lerp(T a, T b, Real t)
{
T p;
p=a+(b-a)*t;
return p;
}



}
#endif



