/*!
* This code has been modified from the lesson below, I have borrowed the
* BMP loader and added it to my image class Jon Macey
*
* NOTE : as we use different Architectures we need to define 
*  -D SOLARIS somewhere if we use the sun boxes else this will
*  not work due to endian differences between INTEL and SPARC
*
*
*
*
 * Simon Werner, 23.07.2000
 * 
 * OpenGL lesson 18. This lesson is the Linux version based on lesson 18 
 * written by NeHe (Jeff Molofee), http://nehe.gamedev.net/ . It is also 
 * based on other ports done by Richard Campbell. This code includes snippets 
 * from lessons 5, 6, 7 and 8. 
 */




#include "Image.h"
#include <stdio.h>
#include <stdlib.h>
//AS SOLARIS IS DIFFERENT ENDIAN THAN INTEL WE NEED TWO
//VERSIONS OF THE LOADER

using namespace std;
using namespace GraphicsLib;
#ifdef SOLARIS
static GLuint getint(FILE *fp)
{
  GLint c, c1, c2, c3;

  // get 4 bytes
  c = getc(fp);
  c1 = getc(fp);
  c2 = getc(fp);
  c3 = getc(fp);
  
  return ((GLuint) c) +   
    (((GLuint) c1) << 8) + 
    (((GLuint) c2) << 16) +
    (((GLuint) c3) << 24);
}

static GLuint getshort(FILE *fp)
{
  GLint c, c1;
  
  //get 2 bytes
  c = getc(fp);  
  c1 = getc(fp);

  return ((GLuint) c) + (((GLuint) c1) << 8);
}

int Image ::loadImage(char *filename)
{
  FILE *file;
	GLuint size;                 // size of the image in bytes.
    GLuint i;                    // standard counter.
    unsigned short int planes;          // number of planes in image (must be 1) 
    unsigned short int bpp;             // number of bits per pixel (must be 24)
    char temp;                          // temporary color storage for bgr-rgb conversion.

    // make sure the file is there.
    if ((file = fopen(filename, "rb"))==NULL) {
      printf("File Not Found : %s\n",filename);
      return 0;
    }

    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // No 100% errorchecking anymore!!!

    // read the width
    sizeX = getint (file);
  //  printf("Width of %s: %lu\n", filename, sizeX);

    // read the height
    sizeY = getint (file);
//    printf("Height of %s: %lu\n", filename, sizeY);

    // calculate the size (assuming 24 bits or 3 bytes per pixel).
    size = sizeX * sizeY * 3;

    // read the planes
    planes = getshort(file);
    if (planes != 1) {
	printf("Planes from %s is not 1: %u\n", filename, planes);
	return 0;
    }

    // read the bpp
    bpp = getshort(file);
    if (bpp != 24) {
	printf("Bpp from %s is not 24: %u\n", filename, bpp);
	return 0;
    }
	
    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    data = (GLubyte *) malloc(size);
    if (data == NULL) {
	printf("Error allocating memory for color-corrected image data");
	return 0;	
    }

    if ((i = fread(data, size, 1, file)) != 1) {
	printf("Error reading image data from %s.\n", filename);
	return 0;
    }

    for (i=0;i<size;i+=3) { // reverse all of the colors. (bgr -> rgb)
	temp = data[i];
	data[i] = data[i+2];
	data[i+2] = temp;
    }
 	fclose(file)  ;
    // we're done.
    return 1;
}
#else
int Image ::loadImage(char *filename)
{

  FILE *file;
    GLuint size;                 // size of the image in bytes.
    GLuint i;                    // standard counter.
    unsigned short int planes;          // number of planes in image (must be 1)
    unsigned short int bpp;             // number of bits per pixel (must be 24)
    char temp;                          // used to convert bgr to rgb color.

	// Make sure the file exists
	if ((file = fopen(filename, "rb"))==NULL)
	{
		printf("File Not Found : %s\n",filename);
		return 0;
	}

	// Skip to bmp header
	fseek(file,18, SEEK_CUR);

	// read width
	if ((i = fread(&sizeX, 4, 1, file)) != 1) {
		printf("Error reading width from %s.\n", filename);
		return 0;
	}
//	printf("Width of %s: %lu\n",filename, sizeX);

	//read the height
	if ((i = fread(&sizeY,4,1,file)) != 1) {
		printf("Error reading height from %s.\n", filename);
		return 0;
	}
	//printf("Height of %s: %lu\n", filename, sizeY);

	// calculate the size (assuming 24 bpp)
	size = sizeX * sizeY * 3;

	// read the planes
	if ((fread(&planes, 2, 1, file)) != 1) {
		printf("Error reading planes from %s. \n", filename);
		return 0;
	}

	if (planes != 1) {
		printf("Planes from %s is not 1: %u\n", filename, planes);
		return 0;
	}

	// read the bpp
	if ((i = fread(&bpp, 2, 1, file)) != 1) {
		printf("Error reading bpp from %s. \n", filename);
		return 0;
	}

	if (bpp != 24) {
		printf("Bpp from %s is not 24: %u\n", filename, bpp);
		return 0;
	}

	// seek past the rest of the bitmap header
	fseek(file, 24, SEEK_CUR);

	// Read the data
	data = (GLubyte *) malloc(size);
	if (data == NULL) {
		printf("Error allocating memory for colour-corrected image data");
		return 0;
	}

	if ((i = fread(data,size,1,file)) != 1) {
		printf("Error reading image data from %s.\n", filename);
		return 0;
	}

	// reverse all of the colours bgr => rgb)
	for (i=0;i<size;i+=3) {
		temp = data[i];
		data[i] = data[i+2];
		data[i+2] = temp;
	}
	fclose(file)  ;
	// Thats all folks
	return 1;

}
#endif

Image :: Image()
{
sizeX=0;
sizeY=0;
data=NULL;

}

Image :: Image(char *fname)
{
sizeX=0;
sizeY=0;
data=NULL;
loadImage(fname);
}


Image::~Image()
{
free(data);
}

GLuint Image::GetFromXY(int x, int y)
{
 return data[x*3+((y)*sizeX*3)];

}

unsigned int Image::setTexture(void)
{
unsigned int textureName;
glGenTextures(1,&textureName);
glBindTexture(GL_TEXTURE_2D,textureName);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);

glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,sizeX,sizeY,0,GL_RGB,GL_UNSIGNED_BYTE,data);
glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
return textureName;
}



