#include "Matrix.h"
#include "Util.h"
#include <GL/gl.h>
#include <iostream>
using namespace std;
using namespace GraphicsLib;
Matrix::Matrix()
{
memset(&m,0,sizeof(m));
}
Matrix::Matrix(const Matrix& M)
{
memcpy(m,&M.m,sizeof(m));
}
Matrix::Matrix(
    		Real m00,Real m01,Real m02, Real m03,
    		Real m10,Real m11,Real m12, Real m13,
            Real m20,Real m21,Real m22, Real m23,
            Real m30,Real m31,Real m32, Real m33
         )
{
m[0][0]=m00; m[0][1]=m01; m[0][2]=m02; m[0][3]=m03;
m[1][0]=m10; m[1][1]=m11; m[1][2]=m12; m[1][3]=m13;
m[2][0]=m20; m[2][1]=m21; m[2][2]=m22; m[2][3]=m23;
m[3][0]=m30; m[3][1]=m31; m[3][2]=m32; m[3][3]=m33;
}


void Matrix::ToReal(Real in[16])
{
GLint i=0;
for(GLint x=0;x<4; x++)
  for(GLint y=0; y<4; y++)
  {
  in[i]=m[x][y];
  i++;
  }
}
void Matrix::Mi(GLint x,GLint y,Real equals)
{
	m[x][y]=equals;
}
void Matrix::null(void)
{
memset(&m,0,sizeof(m));
}
void Matrix::Identity(void)
{
  memset(m,0,sizeof(m));
  m[0][0]=m[1][1]=m[2][2]=m[3][3]=1.0;
}

Matrix Matrix::operator*(const Matrix& m1)
{
Matrix m2;
int i,j,k;
Real ab;
 for(i=0; i<4; i++)
  for(j=0; j<4; j++)
	 {
	  ab=0.0f;
	  for(k=0; k<4; k++)
		   ab+=m[i][k]*m1.m[k][j];
	  m2.m[i][j]=ab;
	}
	  return m2;
}


void Matrix::operator*= (const Matrix &m1)
{
Matrix m2;
int i,j,k;
Real ab;
  for(i=0; i<4; i++)
	for(j=0; j<4; j++)
	{
    ab=0.0f;
	  for(k=0; k<4; k++)
	     ab+=m[i][k]*m1.m[k][j];
		 m2.m[i][j]=ab;
	}
memcpy(m,&m2.m,sizeof(m));
}



Matrix Matrix::operator+(const Matrix &m1)
{
Matrix Ret;
for(int y=0; y<4; y++)
	for(int x=0; x<4; x++)
    	{
        Ret.m[y][x]=m[y][x]+m1.m[y][x];
        }
 return Ret;
}
void Matrix::operator+=(const Matrix &m1)
{
for(int y=0; y<4; y++)
	for(int x=0; x<4; x++)
    	{
        m[y][x]+=m1.m[y][x];
        }

}
Matrix Matrix::operator*(const Real i)
{
Matrix ret;
for(int y=0; y<4; y++)
	for(int x=0; x<4; x++)
    	{
        ret.m[y][x]=m[y][x]*i;
        }
return ret;

}

void Matrix::operator*=(const Real i)
{
for(int y=0; y<4; y++)
	for(int x=0; x<4; x++)
    	{
        m[y][x]*=i;
        }
}



void Matrix::Transpose(void)
{
//Matrix *tmp=this;
Matrix tmp(*this);

for(int row=0; row<4; row++)
	for(int col=0; col<4; col++)
	   {
       m[row][col]=tmp.m[col][row];
       }

}
void Matrix::LoadModelView(void)
{
Real M[16];
this->ToReal(M);
glMatrixMode(GL_MODELVIEW);
glLoadMatrixf(M);
}
void Matrix::LoadProjection(void)
{
Real M[16];
this->ToReal(M);
glMatrixMode(GL_PROJECTION);
glLoadMatrixf(M);
}
void Matrix::RotateX(Real deg)
{
Real cosTheta=cos(DegtoRad(deg));
Real sinTheta=sin(DegtoRad(deg));
m[1][1]=cosTheta; m[1][2]=-sinTheta;
m[2][1]=sinTheta; m[2][2]=cosTheta;
}

void Matrix::RotateY(Real deg)
{
Real cosTheta=cos(DegtoRad(deg));
Real sinTheta=sin(DegtoRad(deg));
m[0][0]=cosTheta; m[0][2]=sinTheta;
m[2][0]=-sinTheta; m[2][2]=cosTheta;
}

void Matrix::RotateZ(Real deg)
{
Real cosTheta=cos(DegtoRad(deg));
Real sinTheta=sin(DegtoRad(deg));
m[0][0]=cosTheta; m[0][1]=-sinTheta;
m[1][0]=sinTheta; m[1][1]=cosTheta;
}



void Matrix::Translate(Real x, Real y, Real z)
{
m[0][3]=x;
m[1][3]=y;
m[2][3]=z;
}
void Matrix::Scale(Real x,Real y, Real z)
{
m[0][0]=x;
m[1][1]=y;
m[2][2]=z;

}

/*! Determinant code modified from 
http://www.euclideanspace.com/maths/algebra/matrix/functions/determinant/index.htm

*/
double  Matrix::Determinant(void)
{
 double value;
    value = 
      m[0][3] * m[1][2] * m[2][1] * m[3][0]-m[0][2] * m[1][3] * m[2][1] * m[3][0]-
      m[0][3] * m[1][1] * m[2][2] * m[3][0]+m[0][1] * m[1][3] * m[2][2] * m[3][0]+
      
      
      m[0][2] * m[1][1] * m[2][3] * m[3][0]-m[0][1] * m[1][2] * m[2][3] * m[3][0]-
      m[0][3] * m[1][2] * m[2][0] * m[3][1]+m[0][2] * m[1][3] * m[2][0] * m[3][1]+
      
      m[0][3] * m[1][0] * m[2][2] * m[3][1]-m[0][0] * m[1][3] * m[2][2] * m[3][1]-
      m[0][2] * m[1][0] * m[2][3] * m[3][1]+m[0][0] * m[1][2] * m[2][3] * m[3][1]+
      m[0][3] * m[1][1] * m[2][0] * m[3][2]-m[0][1] * m[1][3] * m[2][0] * m[3][2]-
      m[0][3] * m[1][0] * m[2][1] * m[3][2]+m[0][0] * m[1][3] * m[2][1] * m[3][2]+
      
      m[0][1] * m[1][0] * m[2][3] * m[3][2]-m[0][0] * m[1][1] * m[2][3] * m[3][2]-
      m[0][2] * m[1][1] * m[2][0] * m[3][3]+m[0][1] * m[1][2] * m[2][0] * m[3][3]+
      
      m[0][2] * m[1][0] * m[2][1] * m[3][3]-m[0][0] * m[1][2] * m[2][1] * m[3][3]-
      m[0][1] * m[1][0] * m[2][2] * m[3][3]+m[0][0] * m[1][1] * m[2][2] * m[3][3];
      return value;

}



ostream& GraphicsLib::operator<<(ostream &output, const GraphicsLib::Matrix &M)
  {
    cout.setf(ios::fixed|ios::adjustfield|ios::showpos);
    cout.precision(2); 
    return output 
    	  <<"["<<M.m[0][0]<<","<<M.m[0][1]<<","<<M.m[0][2]<<","<<M.m[0][3]<<"]"<<endl
    	  <<"["<<M.m[1][0]<<","<<M.m[1][1]<<","<<M.m[1][2]<<","<<M.m[1][3]<<"]"<<endl
          <<"["<<M.m[2][0]<<","<<M.m[2][1]<<","<<M.m[2][2]<<","<<M.m[2][3]<<"]"<<endl
          <<"["<<M.m[3][0]<<","<<M.m[3][1]<<","<<M.m[3][2]<<","<<M.m[3][3]<<"]"<<endl;
  }


void Matrix::Euler(float angle, float x,float y, float z)
{
float c=cos(DegtoRad(angle));
float s=sin(DegtoRad(angle));
 
m[0][0]=c+(1-c)*(x*x); m[0][1]=(1-c)*((y*x)-(s*z)); m[0][2]=(1-c)*((z*x)+(s*y));
m[1][0]=(1-c)*((x*y)+(s*z)); m[1][1]=c+(1-c)*(y*y); m[1][2]=(1-c)*((z*y)-(s*x));
m[2][0]=(1-c)*((x*z)-(s*y)); m[2][1]=(1-c)*((y*z)+(s*x)); m[2][2]=c+(1-c)*(z*z);
}


  
  
  
  

