#include "ObjLoader.h"
#include <stdio.h>
#include <stdlib.h>

using namespace std;
using namespace GraphicsLib;
#define BUFFSIZE 500

// determine what token is used
ObjLoader::ObjToken ObjLoader::ParseString(char *str)
{
// a vertex
if(str[0] =='v' && str[1] ==' ')
	return OVERTEX;
// a texture
else if(str[0] =='v' && str[1]=='t')
	return OTEXTURE;
// a normal
else if(str[0]=='v' && str[1]=='n')
	return ONORMAL;
// a face
else if (str[0] == 'f')
	return OFACE;
else return OUNKNOWN;
}

void ObjLoader::ParseVertex(char *str)
{
char tmp[4];
sscanf(str,"%c %f %f %f\n",&tmp,&Verts[currVert].x,&Verts[currVert].y,&Verts[currVert].z);
currVert++;
}

void ObjLoader::ParseNormal(char *str)
{
char tmp[4];
sscanf(str,"%s %f %f %f\n",tmp,&Norm[currNorm].x,&Norm[currNorm].y,&Norm[currNorm].z);
currNorm++;
}

void ObjLoader::ParseTex(char *str)
{
char tmp[4];
sscanf(str,"%s %f %f \n",tmp,&Tex[currTex].x,&Tex[currTex].y);
currTex++;
}


void ObjLoader::ParseFace(char *str)
{
//first find if it's a tri or a quad
Face[currFace].NumVerts=0;
char temp[BUFFSIZE];
int index=2; //skip 'f '

while(str[index]!='\n')
	if(str[index++] == ' ')
		Face[currFace].NumVerts++;
index=2;

for(unsigned long int i=0; i<=Face[currFace].NumVerts; i++)
{
int j=0;
int strindex=0;
char tt[BUFFSIZE];
// break down face data
while((str[index]!=' ') &&( str[index]!='\n'))
	temp[strindex++]=str[index++];
temp[strindex]='\n'; //best null terminate
unsigned long int ti=0;
//now parse for each face.
//VERTEX
while(temp[ti]!='/')
	tt[j++]=temp[ti++];
tt[j]='\n';
Face[currFace].Vert[i]=(atoi(tt));
Face[currFace].Vert[i]-=1;
//text
j=0;
ti++; // skip /
while(temp[ti]!='/')
	tt[j++]=temp[ti++];
tt[j]='\n';
Face[currFace].Tex[i]=(atoi(tt));
Face[currFace].Tex[i]-=1;
j=0;
ti++; // skip /
//normal
while(temp[ti]!='\n')
	tt[j++]=temp[ti++];
tt[j]='\n';
Face[currFace].Norm[i]=(atoi(tt));
Face[currFace].Norm[i]-=1;
index++; //skip space
}
//Face[currFace].Print();

currFace++;

}



/*! \todo re-write to use c++ stream */


void ObjLoader::Load(char *fname)
{
   FILE *stream;
   char buffer[BUFFSIZE];
    /* Open file and test for successful operation */

    if ((stream = fopen(fname, "rt")) == NULL)
	{
        printf("Failed to open %s file for reading. \n",fname);
        exit(0);
    }
	// first find out how much we have to store.
	do {
		fgets(buffer,BUFFSIZE,stream);

		int type=ParseString(buffer);
		switch (type)
		{
		case OVERTEX : nVerts++; break;
		case ONORMAL : nNorm++; break;
	    case OTEXTURE : nTex++; break;
		case OFACE :   nFaces++; break;
		case OUNKNOWN : /*cout << buffer<<endl;*/ break;

		}
	}while (!feof(stream));

	// now allocate space for each
	//

	Verts= new Point3[nVerts];
	Norm= new Point3[nNorm];
	Tex = new Point3[nTex];
	Face = new ObjFace[nFaces];
	rewind(stream); //back to begining of file;
	// now parse the file for the different entries

	do {
		fgets(buffer,BUFFSIZE,stream);
		int type=ParseString(buffer);
		switch (type)
		{
		case OVERTEX : ParseVertex(buffer); break;
		case ONORMAL : ParseNormal(buffer); break;
	    case OTEXTURE : ParseTex(buffer); break;
		case OFACE :   ParseFace(buffer); break;
		case OUNKNOWN :  break;
		}
	}while (!feof(stream));
	// finally close the file

	fclose(stream);

}

ObjLoader::ObjLoader(char *fname)
{
// set default values
nVerts=nNorm=nTex=nFaces=0;
currFace=currTex=currNorm=currVert=0;
// load the file in
Load(fname);
// create it as a display list
CreateDisplayList();
Texture = NULL;

}

ObjLoader::ObjLoader(char *fname,bool i)
{
// set default values
nVerts=nNorm=nTex=nFaces=0;
currFace=currTex=currNorm=currVert=0;
// load the file in
Load(fname);
// create it as a display list
//CreateDisplayList();
Texture = NULL;
}


ObjLoader::ObjLoader(char *fname,char *fname2)
{
// set default values
nVerts=nNorm=nTex=nFaces=0;
currFace=currTex=currNorm=currVert=0;
// load the file in
Load(fname);
// load texture
LoadTexture(fname2);
// create it as a display list
//CreateDisplayList();

}

void ObjLoader::Draw(void)
{
// draw each of the faces

if(Texture!=NULL)
{
//cout << "setting texture"<<endl;
//glEnable(GL_LIGHTING);
//glEnable(GL_LIGHT0);
glEnable(GL_TEXTURE_2D);
glBindTexture(GL_TEXTURE_2D,Tname);
glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP); //CLAMP_TO_EDGE);
glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP); //CLAMP_TO_EDGE);

}

//glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
for(unsigned long int i=0; i<nFaces; i++)
	{
/*	if(Face[i].inface == true) //true)
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		else
			glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
    */
	// if it is a tri
    if(Face[i].NumVerts == 2)
		glBegin(GL_TRIANGLES);
    else // else its a quad
		glBegin(GL_QUADS);
	// now for each vert in the face
    for(unsigned int x=0; x<=Face[i].NumVerts; x++)
		{
        //find the vert and normal indices
		unsigned long int V=Face[i].Vert[x];
		unsigned long int N=Face[i].Norm[x];
		unsigned long int T=Face[i].Tex[x];
		// and set normal and draw vertex
        //shoudld add texture elements here when required
		glNormal3f(Norm[N].x,Norm[N].y,Norm[N].z);
		if(Texture !=NULL)
			{glTexCoord2f(Tex[T].x,Tex[T].y); /*Tex[T].Print("T");*/}
		glVertex3f(Verts[V].x,Verts[V].y,Verts[V].z);
		}
		glEnd();
	}
if(Texture!=NULL)
{
//glDisable(GL_LIGHTING);
glDisable(GL_TEXTURE_2D);
}
}

void ObjLoader::CreateDisplayList(void)
{
// get a free display list
GLuint range=glGenLists(1);
ListIndex=glGenLists(range);
// create a new list in compile mode so we only create it and not draw it
glNewList(ListIndex,GL_COMPILE);
 // just use the standard draw method as this calls the correct commands
 Draw();
glEndList();
// end the list
}

void ObjLoader::DrawDisplayList(void)
{
// call the list
// note this will draw the object without any transformation so it is best to model all
//obj files at 0,0,0 origin and move them in the program
glCallList(ListIndex);
}
ObjLoader::~ObjLoader()
{
	// tidy up by deleting all the data
    delete  Verts;
	delete  Norm;
	//if(Tex!=NULL)
    	//delete  Tex;
	delete  Face;
}
void ObjLoader::LoadTexture(char *Fname2)
{
// load in the texture
Texture = new Image(Fname2);
Tname=Texture->setTexture();
}


void ObjLoader::ResetInFace(void)
{
for(unsigned long int i=0; i<nFaces; i++)
	Face[i].inface=false;
}


Real ObjLoader::GetHeight(Point3 p)
{
Real y=0;
Real t1,t2,t3,t4=0.0;
Vector b1,b2,b3,b4;
Vector np1,np2,np3,np4;
Point3 nf1,nf2,nf3,nf4;

// loop through all the faces
for(unsigned long int i=0; i<nFaces; i++)
{
// construct the point to test, as were in the X-Z plane with Y being up we reconstruct the
// Point to be in the correct orientation (assuming the point passed in is constructed properly
// from the x-Z value of course ;-)

if(Face[i].NumVerts ==2)
{
b1.set(Verts[Face[i].Vert[1]].x-Verts[Face[i].Vert[0]].x,Verts[Face[i].Vert[1]].z-Verts[Face[i].Vert[0]].z,0);
b2.set(Verts[Face[i].Vert[2]].x-Verts[Face[i].Vert[1]].x,Verts[Face[i].Vert[2]].z-Verts[Face[i].Vert[1]].z,0);
b3.set(Verts[Face[i].Vert[0]].x-Verts[Face[i].Vert[2]].x,Verts[Face[i].Vert[0]].z-Verts[Face[i].Vert[2]].z,0);
}
else
{
// b1.set(Verts[Face[i].Vert[1]].x-Verts[Face[i].Vert[0]].x,Verts[Face[i].Vert[1]].z-Verts[Face[i].Vert[0]].z,0);
// b2.set(Verts[Face[i].Vert[2]].x-Verts[Face[i].Vert[1]].x,Verts[Face[i].Vert[2]].z-Verts[Face[i].Vert[1]].z,0);
// b3.set(Verts[Face[i].Vert[3]].x-Verts[Face[i].Vert[2]].x,Verts[Face[i].Vert[3]].z-Verts[Face[i].Vert[2]].z,0);
// b4.set(Verts[Face[i].Vert[0]].x-Verts[Face[i].Vert[3]].x,Verts[Face[i].Vert[0]].z-Verts[Face[i].Vert[3]].z,0);

b1.set(Verts[Face[i].Vert[0]].x-Verts[Face[i].Vert[1]].x,Verts[Face[i].Vert[0]].z-Verts[Face[i].Vert[1]].z,0);
b2.set(Verts[Face[i].Vert[1]].x-Verts[Face[i].Vert[2]].x,Verts[Face[i].Vert[1]].z-Verts[Face[i].Vert[2]].z,0);
b3.set(Verts[Face[i].Vert[2]].x-Verts[Face[i].Vert[3]].x,Verts[Face[i].Vert[2]].z-Verts[Face[i].Vert[3]].z,0);
b4.set(Verts[Face[i].Vert[3]].x-Verts[Face[i].Vert[0]].x,Verts[Face[i].Vert[3]].z-Verts[Face[i].Vert[0]].z,0);

}
// the point normal is then determined by negeting the y value and swapping it for the x
// in the newly formed vector
np1.set(-b1.y,b1.x,0);//b1.z);
np2.set(-b2.y,b2.x,0);//b2.z);
np3.set(-b3.y,b3.x,0);//b3.z);

// now we calculate the new value for b1 by subtracting the tri point from the point were
// were testing agains
nf1.set(Verts[Face[i].Vert[0]].x,Verts[Face[i].Vert[0]].z,0);
nf2.set(Verts[Face[i].Vert[1]].x,Verts[Face[i].Vert[1]].z,0);
nf3.set(Verts[Face[i].Vert[2]].x,Verts[Face[i].Vert[2]].z,0);

b1=nf1-p; 
b2=nf2-p; 
b3=nf3-p; 
// calc the dot product
t1= b1.x*np1.x+ b1.y*np1.y;
t2= b2.x*np2.x+ b2.y*np2.y;
t3= b3.x*np3.x+ b3.y*np3.y;

if(Face[i].NumVerts ==3)
{
np4.set(-b4.y,b4.x,0);//b3.z);
nf4.set(Verts[Face[i].Vert[3]].x,Verts[Face[i].Vert[3]].z,0);
b4=nf4-p;
t4= b4.x*np4.x+ b4.y*np4.y;
}
// now we take the dot product of each of these values using the hard coded version below
// as we dont care about the Z value


// if the point is within the triangle then the dot product for each line will be <0
if(Face[i].NumVerts ==2)
{
if(t1<=0 && t2<=0 && t3<=0  )
   {
    Face[i].inface=true;
	y=(Verts[Face[i].Vert[0]].y+Verts[Face[i].Vert[1]].y+Verts[Face[i].Vert[2]].y)/3;
	return y;
   }
}

else
{
if(t1<=0 && t2<=0 && t3<=0 &&  t4<=0 )
   {
   Face[i].inface=true;
	y=(Verts[Face[i].Vert[0]].y+Verts[Face[i].Vert[1]].y+Verts[Face[i].Vert[2]].y+Verts[Face[i].Vert[3]].y)/4;
	return y;
   }
}
} //end for
//cout <<"Not Found"<<endl;
return y;

}

void ObjLoader::Scale(Real x, Real y, Real z)
{

for(unsigned long int i =0; i<nVerts; i++)
	{
	Verts[i].x *=x;
	Verts[i].y *=y;
	Verts[i].z *=z;

	}


}
