//--------------------------------------------------------------

#include <GL/gl.h>
#include <GL/glu.h>
#include "SpotLight.h"
#include "Material.h"
//---------------------------------------------------------------------------
using namespace GraphicsLib;
SpotLight::SpotLight(GLint number, Point3 Pos, Point3 Aim, Colour col)
    :Light(number,Pos,col,LIGHTLOCAL)
{
    // set up direction and default values
	Vector Dir(Aim.x-Pos.x,Aim.y-Pos.y,Aim.z-Pos.z);
	Dir.normalize();
	SpotDir[0] = Dir.x;
    SpotDir[1] = Dir.y;
    SpotDir[2] = Dir.z;

    // set some good default values
    CutoffAngle = 45.0;
    SpotExponent = 0.0;

    ConstantAtten = 1.5;
    LinearAtten = 0.0;
    QuadraticAtten = 0.0;
	pos.x=Pos.x; pos.y=Pos.y; pos.z=Pos.z;
	aim.x=Aim.x; aim.y=Aim.y; aim.z=Aim.z;
	dir.x=Dir.x; dir.y=Dir.y; dir.z=Dir.z;
}




SpotLight::SpotLight(GLint number, Point3 Pos, Vector Dir, Colour col)
    :Light(number,Pos,col,LIGHTLOCAL)
{
    // set up direction and default values
    Dir.normalize();
    SpotDir[0] = Dir.x;
    SpotDir[1] = Dir.y;
    SpotDir[2] = Dir.z;

    // set some good default values
    CutoffAngle = 45.0;
    SpotExponent = 0.0;
    ConstantAtten = 1.5;
    LinearAtten = 0.0;
    QuadraticAtten = 0.0;
}

void SpotLight::Show(void)
{
    // call superclass show method
    Light::Show();
	this->Enable();
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
		glVertex3f(pos.x,pos.y,pos.z);
		glVertex3f(aim.x,aim.y,aim.z);
	glEnd();
	glEnable(GL_LIGHTING);
	// change spot light direction
    glLightfv(GL_LIGHT0+LightNo, GL_SPOT_DIRECTION, SpotDir);

}
void SpotLight::Aim(Point3 Pos)
{
// dir.x=pos.x-Pos.x;
// dir.y=pos.y-Pos.y;
// dir.z=pos.z-Pos.z;
Vector D= Pos-pos;
D.normalize();
SpotDir[0] = D.x;
SpotDir[1] = D.y;
SpotDir[2] = D.z;

glLightfv(GL_LIGHT0+LightNo, GL_SPOT_DIRECTION, SpotDir);

}



void SpotLight::Enable()
{

	int lightNum = GL_LIGHT0+LightNo; //grab defined int for GL's light0

    glLightf(lightNum, GL_CONSTANT_ATTENUATION, ConstantAtten);
    glLightf(lightNum, GL_LINEAR_ATTENUATION, LinearAtten);
    glLightf(lightNum, GL_QUADRATIC_ATTENUATION, QuadraticAtten);

    glLightf(lightNum, GL_SPOT_CUTOFF, CutoffAngle);
    glLightfv(lightNum, GL_SPOT_DIRECTION, SpotDir);
    glLightf(lightNum, GL_SPOT_EXPONENT, SpotExponent);
  	Light::Enable();
    // call base class enable

}

void SpotLight::SetParams( Real Cutoff,Real Exponent,
			   Real Constant, Real Linear,
			   Real Quadratic)
{
CutoffAngle=Cutoff;
SpotExponent=Exponent;
ConstantAtten=Constant;
LinearAtten=Linear;
QuadraticAtten=Quadratic;
}


