
#ifdef __USINGWINDOWS__
	#include <windows.h>
#endif

#include "Point3.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include "Vector.h"
#include <time.h>
#include <stdlib.h>

 #define TWO_PI 6.28318530717958647692   //360
 #define PI 3.14159265358979323846       //180
 #define PI2 1.57079632679489661923      //90
 #define PI4 0.785398163397448309615     //45
//
//

namespace GraphicsLib {


void CreateAxis(void)
{

glBegin(GL_LINES); //x
    glColor3f(1.0f,0.0f,0.0f);
    glVertex3f(-0.8f,0.0f,0.0f);
    glVertex3f(0.8f,0.0f,0.0f);
    glColor3f(0.0f,1.0f,0.0f); //y
    glVertex3f(0.0f,-0.8f,0.0f);
    glVertex3f(0.0f,0.8f,0.0f);
    glColor3f(0.0f,0.0f,1.0f); //z
    glVertex3f(0.0f,0.0f,-0.8f);
    glVertex3f(0.0f,0.0f,0.8f);
glEnd();
//cones for y
glPushMatrix();
   glTranslatef(0.8f,0.0f,0.0f);
   glRotatef(90.0f,0,1,0);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();
glPushMatrix();
   glTranslatef(-0.8f,-0.0f,0.0f);
   glRotatef(-90.0f,0,1,0);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();

//cones for x
glColor3f(0.0f,1.0f,0.0f);
glPushMatrix();
   glTranslatef(0.0f,0.8f,0.0f);
   glRotatef(-90.0f,1,0,0);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();
glPushMatrix();
   glTranslatef(0.0f,-0.8f,0.0f);
   glRotatef(90.0f,1,0,0);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();

glColor3f(0.0f,0.0f,1.0f);
// cones for z
glPushMatrix();
   glTranslatef(0.0f,0.0f,0.8f);
   glRotatef(-90.0f,0,0,1);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();
glPushMatrix();
   glTranslatef(0.0f,0.0f,-0.8f);
   glRotatef(180.0f,1,0,0);
   glutSolidCone(0.01f,0.04f,8,8);
glPopMatrix();


}

/*!DrawAxis draws a 3d Axis at the current position
 * Weere the axis are represented as follows <BR>
X=RED
Y=GREEN
Z=BLUE
* \param Real Size the scale factor for the axis
*/
void DrawAxis(Real Size)
{
glPushMatrix();
  glScalef(Size,Size,Size);
  CreateAxis();
glPopMatrix();
}

/*!DrawSquare draws a square centered at x,y of size/2
 * \param Real x the x position
 * \param Real y the y position
 * \param Real size the size of the square
 */
void DrawSquare(Real x, Real y,Real size)
{
	glBegin(GL_LINE_LOOP);
        glVertex2f(x-size/2.0f,y+size/2.0f);
        glVertex2f(x+size/2.0f,y+size/2.0f);
        glVertex2f(x+size/2.0f,y-size/2.0f);
        glVertex2f(x-size/2.0f,y-size/2.0f);
        glVertex2f(x-size/2.0f,y+size/2.0f);
	glEnd();
}


/*calculates the normal from 3 points and return the new
 * normal as a Vector
 * \param Point3 p1 the first point
 * \param Point3 p2 the second point
 * \param Point3 p4 the third point
 * \return Vector the normal of the 3 points
 */


Vector CalcNormal ( Point3 p1, Point3 p2, Point3 p3 )
{
  Real coa, cob, coc ;
  Real px1, py1, pz1 ;
  Real px2, py2, pz2 ;
  Real px3, py3, pz3 ;
  Real absvec;
  Vector norm;
  px1 = p1.x; py1 = p1.y; pz1 = p1.z;
  px2 = p2.x; py2 = p2.y; pz2 = p2.z;
  px3 = p3.x; py3 = p3.y; pz3 = p3.z;

// generate cross product

  coa = -(py1 * (pz2-pz3) + py2*(pz3-pz1) + py3*(pz1-pz2)) ;
  cob = -(pz1 * (px2-px3) + pz2*(px3-px1) + pz3*(px1-px2)) ;
  coc = -(px1 * (py2-py3) + px2*(py3-py1) + px3*(py1-py2)) ;

// normalise the results to get unit length normal
  absvec = sqrt ((double) ((coa*coa) + (cob*cob) + (coc*coc))) ;

  norm.x = coa/absvec ;
  norm.y = cob/absvec ;
  norm.z = coc/absvec ;
  norm.w=0.0f;
  return norm;
}



void CreateAxisSimple(void)
{

glPushMatrix();
    glBegin(GL_LINES); //x
        glColor3f(1.0f,0.0f,0.0f);
        glVertex3f(-0.8f,0.0f,0.0f);
        glVertex3f(0.8f,0.0f,0.0f);
        glColor3f(0.0f,1.0f,0.0f); //y
        glVertex3f(0.0f,-0.8f,0.0f);
        glVertex3f(0.0f,0.8f,0.0f);
        glColor3f(0.0f,0.0f,1.0f); //z
        glVertex3f(0.0f,0.0f,-0.8f);
        glVertex3f(0.0f,0.0f,0.8f);
    glEnd();
glPopMatrix();
}

/*! draws a simple axis with no cones
\param Real Size the size of the axis to draw */
void DrawAxisSimple(Real Size)
{
glPushMatrix();
  glScalef(Size,Size,Size);
  CreateAxisSimple();
glPopMatrix();
}

/*! DegtoRad converts degrees to raduans
\param Real deg the angle to convert
\param the angle in radians */
Real DegtoRad(Real deg)
{
    return (deg/360.0f) * TWO_PI;
}
/*! RadtoDeg converts radians to degrees
\param Real rad the angle in radians
\returns the angle in degrees */
Real RadtoDeg(Real rad)
{
    return (rad / TWO_PI) * 360.0f;
}


const int HALF_RAND = (RAND_MAX / 2);
/*! returns a random number between +/- 1
\returns a random number between +/- 1 */
Real RandomNum()
{
	int rn;
	rn = rand();
	return ((Real)(rn - HALF_RAND) / (Real)HALF_RAND);
}
/*! returns a random number between +/- MaxVal
\param Real MaxVal the value to set for the max rand num
\returns a random number between +/- 1 */
Real RandomNum(Real MaxVal)
{
    return MaxVal * RandomNum();
}
/*! returns a positive random number between 0 and MaxVal
\param Real MaxVal the value to set for the max rand num
\returns a random number between 0 and MaxVal */
Real RandomPosNum(Real MaxVal)
{
 return  (MaxVal *rand()/(RAND_MAX+1.0f));
}

/*! set the random seed */

void SetSeed(void)
{
srand(time(NULL));
}


}



