
#include "Vector.h"
#pragma pack(16)
#include "myassert.h"
using namespace GraphicsLib;


void Vector::Normal(void)
{
glNormal3f(x,y,z);
}

void Vector::Vertex(void)
{
glVertex3f(x,y,z);
}

void Vector::Translate(void)
{
glTranslatef(x,y,z);
}

Real Vector::dot(Vector b) // return this dotted with b
{
return x * b.x + y * b.y + z * b.z;
}


void Vector::set(Real nx, Real ny, Real nz, Real nw)
{
   x=nx; y=ny; z=nz; w=nw;
}

void Vector::set(const Vector& vin)
{
   x=vin.x; y=vin.y; z=vin.z;w=vin.w;
}
void Vector::set(const Vector* vin)
{
   x=vin->x; y=vin->y; z=vin->z;w=vin->w;
}


	// initialize from 3 or 4 Reals
void Vector::null(void)
{ 
   x=y=z=0.0f; w=1.0f;
}
// indexed component
Real& Vector::operator[](GLint i) 
{ 
assert(i >=0 || i<=4);
 return (&x)[i]; 
}

Real Vector::length(void)	
{ 
 return (Real)sqrt(x*x+y*y+z*z); 
}

void Vector::negate(void)
{ 
x=-x; y=-y; z=-z; 
}
	
Vector Vector::operator-()
{ 
return Vector(-x,-y,-z,w); 
}
// make vector length one 
void Vector::normalize(void)
{
	Real len=(Real)sqrt(x*x+y*y+z*z);
assert(len!=0);
x/=len; y/=len; z/=len;
}

void Vector::cross(const Vector& v1,const Vector& v2)
{
x=v1.y*v2.z-v1.z*v2.y;
y=v1.z*v2.x-v1.x*v2.z;
z=v1.x*v2.y-v1.y*v2.x;
}

Vector Vector::cross(const Vector b) //return this cross b
{
Vector c(y*b.z - z*b.y, z*b.x - x*b.z, x*b.y - y*b.x,0.0);
return c;
}


void Vector::operator+=(const Vector& v)
{
x+=v.x; y+=v.y; z+=v.z; w+=v.w;
}
void Vector::operator-=(const Vector& v)
{
x-=v.x; y-=v.y; z-=v.z; w-=v.w;
}

void Vector::operator*=(const Vector& v)
{
x*=v.x; y*=v.y; z*=v.z; w*=v.w;
}


void Vector::operator/=(const Vector& v)
{
x/=v.x; y/=v.y; z/=v.z; w/=v.w;
}

Vector Vector::operator/(Real v)
{	
return Vector(x/v,y/v,z=z/v,w);	
}



Vector Vector::operator+(const Vector &v)
{
 return Vector(x+v.x, y+v.y,z+v.z,w);
}



Vector Vector::operator-(const Vector& v)
{
return Vector(x-v.x,y-v.y,z-v.z,w);
}

bool Vector::operator==(const Vector& v)
{
return (v.x == x  && v.y == y && v.z == z);
}

bool Vector::operator!=(const Vector& v)
{
return (v.x != x  || v.y != y || v.z != z);
}


Vector Vector::operator*(const Vector &v)
{
return Vector(x*v.x, y*v.y,z*v.z,w);
}

Vector Vector::operator/(const Vector& v)
{
 return Vector(x/v.x,y/v.y,z/v.z,w);
}

Vector Vector::operator *(Real i)
{
return Vector(x*i,y*i,z*i,w);
}

GLdouble Vector::AngleBetween(const Vector &rhs)const
{
 // uses cos(t) = |a| . |b|
// create tmp vectors so we don't have to 
// normalize existing ones
Vector v1 = rhs;
Vector v2 = *this;
v1.normalize();
v2.normalize();
return acos(v1.dot(v2));
}

double Vector::inner(const Vector& vector) 
{
 return (x * vector.x) + (y * vector.y) + (z * vector.z);
}

Vector Vector::outer(const Vector& vector) 
{
  Real xx = (y * vector.z) - (z * vector.y);
  Real yy = (z * vector.x) - (x * vector.z);
  Real zz = (x * vector.y) - (y * vector.x);

  return Vector(xx,yy,zz);
}


Vector & Vector::operator=(const Vector &rhs) 
{
x = rhs.x; y = rhs.y; z = rhs.z; w=rhs.w;
return *this;
}




std::ostream& GraphicsLib::operator<<(std::ostream& output, const GraphicsLib::Vector& s)
{
    return output<<"["<<s.x<<","<<s.y<<","<<s.z<<","<<s.w<<"]";
}
std::istream& operator>>(std::istream& input, GraphicsLib::Vector& s)
{
return input >> s.x >> s.y >> s.z>>s.w;	
}

std::ostream& GraphicsLib::operator<<(std::ostream& output, const GraphicsLib::Vector* s)
{
    return output<<"["<<s->x<<","<<s->y<<","<<s->z<<","<<s->w<<"]";
}
std::istream& operator>>(std::istream& input, GraphicsLib::Vector *s)
{
return input >> s->x >> s->y >> s->z>>s->w;	
}



Vector Vector::operator*(const Matrix &m)
{
Real *f=(Real *)&m;
return Vector(
			x*f[0] + y*f[4] + z*f[8] + w*f[12],
			x*f[1] + y*f[5] + z*f[9] + w*f[13],
			x*f[2] + y*f[6] + z*f[10] + w*f[14],
			x*f[3] + y*f[7] + z*f[11] + w*f[15]
			);
}

