#ifndef AI_H_INCLUDED
#define AI_H_INCLUDED

#include <set>
#include <map>
#include <cstdlib>
#include <vector>
#include <iostream>
#include <string>
#include <GL/glut.h>
#include "GraphicsLib.h"


#include "botclass.h"
#include "path.h"
#include "map.h"
#include "astar.h"


#define PRINTAI 0
#define DRAWFORCE 0

using namespace std;
using namespace GraphicsLib;

extern Colour colourList[6];

class bot;
class path;
class dataMap;

class AIthought;
    class AIcluster;
	class seekBot;
	class seekPoint;
	class fleeBot;
	class fleePoint;
	class arrival;
	class seperation;
	class cohesion;
	class alignment;
	class followPath;
	class shortestRoute;
	    class AstarNode;

typedef pair <int , AIthought*> AIthoughtList_Pair;
typedef multimap <int , AIthought*>::iterator AIthoughtList_Iter;

typedef vector <AstarNode*>::iterator AstarList_Iter;

void removeAI(AIcluster* whatCluster, AIthoughtList_Iter whatAI_Iter);

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//AIthought - ai superclass
class AIthought {
   public:
	float weight; //0-1 how much influence this AI has
	int activityState; // 0=Active 1=Success 2=Failure
	int priority; //0=always done. 1->x = do only the highest priority, ignore all the rest
	string AIname; //just what this type of AI module is called

	AIcluster* pCluster; //the cluster that this thought is in
	AIthoughtList_Iter Cluster_Iter; //Where in the thoughtlist this thought is

	virtual void action() = 0;
	virtual void success() = 0;
	virtual void failure() = 0;
	virtual int updateActivity() = 0;

	virtual ~AIthought() {};
};


//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
// the AIcluster:
//this stores a list of thoughts and its action runs all the actions of its AI list
// the cool thing is that it is a thought itself - therefore it can be added to another AIcluster, making multiple trains of thought
class AIcluster : public AIthought {
    public:

    	bot* pBot; //the bot that this cluster will be affecting
	int clusterLvl;

	multimap <int , AIthought*> AIthoughtList; //list of pointers to all the AIthoughts that the cluster has currently - indexed by priority
	int maxPriority;

	//TODO class that stores the mentality that this cluster is run by

	//basic AI methods to be performed on the thought list
	void printAIlist();

	void AIaction(AIthought *pWhatThought);
	void decideThoughts(); //what thoughts to add to the AIcluster when it is generated

	//all the methods to add a thought to the list
	AIcluster* addAI_cluster(bot* pBot_in, float weight_in, int priority_in, int activityState_in);
	seekBot* addAI_SeekBot(bot* ptargetBot_in, float t_in, float failureDistance_in, bool flee_in, float weight_in, int priority_in, int activityState_in);
	seekPoint* addAI_SeekPoint(Vector target_in, float failureDistance_in, bool flee_in, float weight_in, int priority_in, int activityState_in);
	seperation* addAI_Seperation(float weight_in, int priority_in, int activityState_in);
	cohesion* addAI_Cohesion(float weight_in, int priority_in, int activityState_in);
	alignment* addAI_Alignment(float weight_in, int priority_in, int activityState_in);
	shortestRoute* addAI_shortestRoute(Vector start_in, Vector target_in, dataMap *ptheMap_in, int numSteps_in, float weight_in, int priority_in, int activityState_in);
	followPath* addAI_followPath(path* whatPath, float radius_in, bool dirInvert_in, float weight_in, int priority_in, int activityState_in);

	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this seek succeeds
	virtual void failure(); //what to do when this seek fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~AIcluster();
};


//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

class seekBot : public AIthought {
   public:
	bot *targetBot; //the bot to seek
	float t; //how far in the future you want to aim for
	float failureDistance; //how far away we let it get before we fail
	bool flee; //0=seek, 1=flee

	float distance; //how far away the target is

	void calcDistance(); // set distance to be how far the bot is from the target
	void drawFailDist();

	// all the following methods are required for an AI module
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this seek succeeds
	virtual void failure(); //what to do when this seek fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~seekBot() {};
};

//*********************************************************************************

class seekPoint : public AIthought {
   public:
	Vector target; //the point to seek
	float failureDistance;
	bool flee; //0=seek, 1=flee

	float distance; //how far away the target is

	void calcDistance(); // set distance to be how far the bot is from the target
	void drawFailDist();

	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this seek succeeds
	virtual void failure(); //what to do when this seek fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~seekPoint() {};
};

//*********************************************************************************

class seperation : public AIthought {
   public:

	Vector seperation_force(bot *pNearBot);

	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this succeeds
	virtual void failure(); //what to do when this fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~seperation() {};
};

//*********************************************************************************

class cohesion : public AIthought {
   public:
	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this succeeds
	virtual void failure(); //what to do when this fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~cohesion() {};
};

//*********************************************************************************

class alignment : public AIthought {
   public:
	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this succeeds
	virtual void failure(); //what to do when this fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~alignment() {};
};

//*********************************************************************************

class followPath : public AIthought {
   public:
	path *thePath; //list of points that the bot must follow
	bool dirInvert; //0=first to last path points  1=last to first path points

	int currPointID;
	//TODO a much more complicated version is needed - for the future

	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this succeeds
	virtual void failure(); //what to do when this fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~followPath() {};
};

//*********************************************************************************

class shortestRoute : public AIthought {
   public:
	Vector start, target;
	int startX, startY;
	int targetX, targetY;
	dataMap *theMap; //the map to be used
	float mapHorizantalDist, mapVerticalDist, mapDiagonalDist;

	vector <AstarNode*> openList;
	vector <AstarNode*> closedList;
	int numSteps; //the number of a* steps performed in one action cycle. 0=do all in one go

	path *thePath; //list of points that the bot must follow

	//methods:
	void InitAstar();
	void AstarStep();

	void moveList(AstarList_Iter fromWhere, vector <AstarNode*> *fromList, vector <AstarNode*> *toList);
	AstarList_Iter newToList(int x, int y, AstarNode* parentNode, vector <AstarNode*> *whatList);
	void removeNode(AstarNode* whatNode, AstarList_Iter nodeIter);
	AstarList_Iter findInList(int x, int y, vector <AstarNode*> *whatList);

	void printOpenList();
	void printClosedList();

	bool isGoal(AstarNode* currNode);

	float heuristic(AstarNode* currNode);
	float calcCost(AstarNode* startNode, AstarNode* endNode);
	void calcRoute(int nodeX, int nodeY);

	void drawList(vector <AstarNode*> *whatList);
	void drawNode(AstarNode* whatNode);

	// all the following methods are required for a
	virtual void action(); //what to do on each update
	virtual void success(); //what to do when this succeeds
	virtual void failure(); //what to do when this fails
	virtual int updateActivity(); //criteria for altering the activity state - whether its a success or a failure

	virtual ~shortestRoute() {};
};

#endif
