#include "astar.h"

//*********************************************************************************
//shortestRoute methods

shortestRoute* AIcluster::addAI_shortestRoute(Vector start_in, Vector target_in, dataMap *ptheMap_in, int numSteps_in, float weight_in, int priority_in, int activityState_in) {

	AIthoughtList_Iter tempIter;
	tempIter = AIthoughtList.insert( AIthoughtList_Pair(priority_in  , new shortestRoute()) ); // add a pointer to the AI list

	//set the attributes for the new seekbot ai element
	shortestRoute* tempShortestRoute = dynamic_cast<shortestRoute*>(tempIter->second);

	tempShortestRoute->pCluster = this;
	tempShortestRoute->Cluster_Iter = tempIter;

	tempShortestRoute->priority = priority_in;
	if (priority_in > maxPriority) maxPriority = priority_in; //set the max priority if the new one is greater

	if (clusterLvl == 0) tempShortestRoute->weight = weight_in;
  	else tempShortestRoute->weight = weight_in * weight;

	tempShortestRoute->activityState = activityState_in;
	tempShortestRoute->AIname = "SHORTEST_ROUTE";

	tempShortestRoute->start = start_in;
	tempShortestRoute->target = target_in;

	tempShortestRoute->theMap = ptheMap_in;
	tempShortestRoute->numSteps = numSteps_in;

	tempShortestRoute->InitAstar();

	cout << "BOT[" << pBot->universeID << "]: Adding SHORTEST_ROUTE ai element - priority:" << priority_in << endl;

	return tempShortestRoute;
}


void shortestRoute::action() {
	cout << "  Doing shortest Route ";
	if (numSteps!=0) {
		cout << numSteps << " steps at a time:" << endl;
		for (int i=0; i<numSteps && activityState==0; i++) {
			//cout << "   > step "<< i+1 << endl;
			AstarStep(); //do one step of the a star program
			//drawList(&closedList);
		}
	} else {
		cout << "all at once:" << endl;
		while (activityState==0) {
			AstarStep();
		}
	}
}

void shortestRoute::success() {
	pCluster->addAI_followPath(thePath, theMap->dataResWidth/2, 0, 1, 1, 0);
	activityState = 3;
	//removeAI(pCluster, Cluster_Iter); // remove this AI module
}

void shortestRoute::failure() {

}

int shortestRoute::updateActivity() {
	return activityState;
}

float shortestRoute::calcCost(AstarNode* startNode, AstarNode* endNode) {
//cost of getting from startNode to endNode
//TODO this function could be in the map or map type class, so it is specific to the map type and
float edgeLength;
	if (startNode->x==endNode->x && startNode->y!=endNode->y) {
		edgeLength = mapVerticalDist;
	} else if (startNode->x!=endNode->x && startNode->y==endNode->y) {
		edgeLength = mapHorizantalDist;
	} else if (startNode->x!=endNode->x && startNode->y!=endNode->y) {
		edgeLength = mapDiagonalDist;
	}

	if (theMap->getValue(startNode->x, startNode->y) != 0 && theMap->getValue(endNode->x, endNode->y) != 0) {
		//for each node that it travels through work out the cost - only really works for maxspeed maps
		return (edgeLength/2)/theMap->getValue(startNode->x, startNode->y) + (edgeLength/2)/theMap->getValue(endNode->x, endNode->y);
	} else return 999;
}

float shortestRoute::heuristic(AstarNode* currNode) {
	float xDist = targetX*mapHorizantalDist - currNode->x*mapHorizantalDist;
	float yDist = targetY*mapVerticalDist - currNode->y*mapVerticalDist;
	//TODO should be in the map class
	//return the straight line distance from node to target
	return sqrt(xDist*xDist + yDist*yDist)/pCluster->pBot->pBotType->max_speed;
}

bool shortestRoute::isGoal(AstarNode* currNode) {
	if(currNode->x == targetX && currNode->y==targetY) return true;
	else return false;
}


void shortestRoute::printOpenList() {
	AstarList_Iter tempIter ;

	cout << "\n---------- Open List ----------" << endl;
	cout <<   "start: " << startX << "," << startY << "  target: " << targetX << "," << targetY << endl;

	for (tempIter=openList.begin(); tempIter!= openList.end(); tempIter++) {
		cout << " > f:" << (*tempIter)->f << " pos: "<< (*tempIter)->x <<","<< (*tempIter)->y << " g:"<< (*tempIter)->g << " h:" << (*tempIter)->h;
		cout << "   parentNode: [" << (*tempIter)->parentX << "," << (*tempIter)->parentY << "]" << endl;
	}
	cout << "Open List total: "<< openList.size() << endl;
}

void shortestRoute::printClosedList() {
	AstarList_Iter tempIter ;

	cout << "\n---------- Closed List ----------" << endl;
	cout <<   "start: " << startX << "," << startY << "  target: " << targetX << "," << targetY << endl;

	for (tempIter=closedList.begin(); tempIter!= closedList.end(); tempIter++) {
		cout << " > f:" << (*tempIter)->f << " pos: "<< (*tempIter)->x <<","<< (*tempIter)->y << " g:"<< (*tempIter)->g << " h:" << (*tempIter)->h;
		cout << "   parentNode: [" << (*tempIter)->parentX << "," << (*tempIter)->parentY << "]" << endl;
	}
	cout << "Closed List total: "<< closedList.size() << endl;
}

void shortestRoute::removeNode(AstarNode* whatNode, AstarList_Iter nodeIter) {

	//cout << "deleting pointer" << endl;
	delete whatNode;

	if (whatNode->list==0) {
		openList.erase(nodeIter);
		//cout << "REMOVING NODE [" << whatNode->x << "," << whatNode->y <<"] from open List" << endl;
	} else {
		closedList.erase(nodeIter);
		//cout << "REMOVING NODE [" << whatNode->x << "," << whatNode->y <<"] from closed List" << endl;
	}
}



AstarList_Iter shortestRoute::findInList(int x, int y, vector <AstarNode*> *whatList) {

	AstarList_Iter tempIter;

	for (tempIter = whatList->begin(); tempIter!=whatList->end(); tempIter++) {
		if ((*tempIter)->x == x && (*tempIter)->y == y) return tempIter;
	}
	return tempIter;
}

void shortestRoute::moveList(AstarList_Iter fromWhere, vector <AstarNode*> *fromList, vector <AstarNode*> *toList) {

	AstarList_Iter tempIter = toList->begin();

	    //cout << "trying to move Node..." << endl;

	    while (tempIter!=toList->end()) {
		//step through every element from begining to end
		//cout << "  >stepping through the list insertNode f:" << (*fromWhere)->f << " ListNode f:" << (*tempIter)->f << endl;
		if ((*tempIter)->f >= (*fromWhere)->f) {
			//cout << "inserting the node newNode" << endl;
			//insert the tempNode before the element that has an f which is greater the tempNode's f
			toList->insert(tempIter, (*fromWhere));
			break; //break out of this while loop
		}
		tempIter++;
	    }
	    if (tempIter==toList->end()) {
	    	toList->insert(tempIter, (*fromWhere));
	    }
	    	if (toList == &openList) (*fromWhere)->list = 0; else (*fromWhere)->list = 1;
		//cout << "\n     Adding Node[" << (*fromWhere)->x << "," << (*fromWhere)->y << "] f:"<< (*fromWhere)->f << " g:" << (*fromWhere)->g << " h:"<< (*fromWhere)->h << "  parentNode["<< (*fromWhere)->parentX << "," << (*fromWhere)->parentY << "]" << endl;

		fromList->erase(fromWhere);

}

AstarList_Iter shortestRoute::newToList(int x, int y, AstarNode* parentNode, vector <AstarNode*> *whatList) {
// create a new node and put it on the open list
    AstarNode* tempNode = new AstarNode;
	tempNode->x = x;
	tempNode->y = y;
	tempNode->value = theMap->getValue(x,y);

	//tempNode->parentNode = parentNode;


	if (parentNode!=NULL) {
		tempNode->parentX = parentNode->x;
		tempNode->parentY = parentNode->y;
		tempNode->g = parentNode->g + calcCost(parentNode, tempNode);
	} else {
		tempNode->parentX = x;
		tempNode->parentY = y;
		tempNode->g = 0;
	}

	tempNode->h = heuristic(tempNode);
	tempNode->f = tempNode->g + tempNode->h;
	if (whatList==&openList) tempNode->list=0;
	else tempNode->list=1;

	//tempNode->listID = whatList->size();
	//whatList->push_back(tempNode);



	//TODO - could be a lot more optimized with a different container multiset or multimap
	AstarList_Iter tempIter = whatList->begin();

	if (parentNode!=NULL) {
	   // cout << "trying to insert into List..." << endl;

	    while (tempIter!=whatList->end()) {
		//step through every element from begining to end
		//cout << "  >stepping through the list insertNode f:" << tempNode->f << " ListNode f:" << (*tempIter)->f << endl;
		if ((*tempIter)->f >= tempNode->f) {
			//cout << "inserting the node newNode" << endl;
			//insert the tempNode before the element that has an f which is greater the tempNode's f
			whatList->insert(tempIter, tempNode);
			break; //break out of this while loop
		}
		tempIter++;
	    }
	    if (tempIter==whatList->end()) {
	    	whatList->insert(tempIter, tempNode);
	    }
		//cout << "\n     Adding Node[" << x << "," << y << "] f:"<< tempNode->f << " g:" << tempNode->g << " h:"<< tempNode->h << "  parentNode["<< tempNode->parentX << "," << tempNode->parentY << "]" << endl;
	} else {
		whatList->insert(tempIter, tempNode); //add it to the head if it is the start node
			//cout << "\n     Adding Node[" << x << "," << y << "] f:"<< tempNode->f << " g:" << tempNode->g << " h:"<< tempNode->h << "  parentNode[NULL]" << endl;
	}

	return tempIter;
}

void shortestRoute::InitAstar() {
//start up the a* algorithm
	//setup all the attributes of the a* thought
	Vector tempVec = theMap->nearestMAPcoord(start.x, start.y);
	startX = (int)tempVec.x;
	startY = (int)tempVec.y;

	tempVec = theMap->nearestMAPcoord(target.x, target.y);
	targetX = (int)tempVec.x;
	targetY = (int)tempVec.y;

	mapHorizantalDist = theMap->dataResWidth;
	mapVerticalDist = theMap->dataResHeight;
	mapDiagonalDist = sqrt((mapHorizantalDist*mapHorizantalDist) + (mapVerticalDist*mapVerticalDist));
	cout << "  Initialise A* thought-> start:"<< start << "(" << startX << "," << startY << ") target:"<< target << "(" << targetX << "," << targetY << ")" << endl;
	openList.clear();
	openList.reserve(theMap->dataWidth*theMap->dataHeight);
	closedList.clear();
	closedList.reserve(theMap->dataWidth*theMap->dataHeight);
	//TODO need to sort out the lists - every time a new node is inserted all the pointers go screwy and change.
	// reserving space means that the pointers don't have to change ... maybe
	newToList(startX, startY, NULL, &openList); //add the start position to the openList

}

void shortestRoute::AstarStep() {
//do one step of the a* algorithm
//this is done until the top of the openlist == goal

   //currentNode = openList[0] -> this is the node with the lowest f;
   AstarNode *currentNode = (*openList.begin());
   	//cout << "\n\nAI STEP: currentNode [" << currentNode->x << "," << currentNode->y << "]" << endl;
	moveList(openList.begin(), &openList, &closedList);
	//openList.erase(openList.begin());

   AstarList_Iter tempIter;


   if (isGoal(currentNode)) {
	//if currentNode is goal then it is done. now find route backwards
	cout << "   Found the goal" << endl;
	thePath = new path;
	thePath->pTheUniverse = pCluster->pBot->pTheUniverse;
   	calcRoute(currentNode->x, currentNode->y);
   } else {


	//make all the node successors -> all the nodes around the current node except the parent node and nodes that are outside the map
	AstarNode successor[7];
	for (int i=0; i<8; i++) {
	//step through all the node successors:
		successor[i].x = currentNode->x;
		successor[i].y = currentNode->y;
		//set the x & y values of all the successors
		if (i>=2 && i<=4) successor[i].x++;
		if (i==6 || i==7 || i==0) successor[i].x--;
		if (i>=0 && i<=2) successor[i].y++;
		if (i>=4 && i<=6) successor[i].y--;

		//cout << "   successor[" << i << "] x:" << successor[i].x <<" y:" << successor[i].y;
		if ((successor[i].x >= 0 && successor[i].x < theMap->dataWidth) && (successor[i].y >= 0 && successor[i].y < theMap->dataHeight)) {
		//its not the parentNode and its within the map bounds
			//cout << " is within the map";
		   if (!(successor[i].x==currentNode->parentX && successor[i].y==currentNode->parentY)) {
			//cout << " & is not the parent" << endl;
			//calculate the f,g,h of the successor
			successor[i].g = currentNode->g + calcCost(currentNode, &successor[i]);
			successor[i].h = heuristic(&successor[i]);
			successor[i].f = successor[i].g + successor[i].h;

			//if successor's position is on the openList but successor is worse then ignore this successor
			//if successor's position is on the closedList but successor is worse then ignore this successor

			//if its better than the occurances on the open and closed list then remove all versions from open and closed
			bool isBetter = true;
			//cout << "finding node in open List" << endl;
			for (tempIter = openList.begin(); tempIter != openList.end() && isBetter!=false; tempIter++) {
				//step though every element in the openList
				if ((*tempIter)->x==successor[i].x && (*tempIter)->y==successor[i].y) {
					//successor is an openList element
					if((*tempIter)->f <= successor[i].f) {
						//the OpenList element is better than the successor
						//as soon as a node is found that is better then stop using this successor
						//cout << "   There is a better version in the openList" << endl;
						isBetter = false;
					} else {
						//the OpenList element is worse
						//remove all occurances that are worse than the successor
						//cout << "   Deleting worse version from openList" << endl;
						isBetter = true;
						removeNode(*tempIter, tempIter);
						break;
					}
				}
			}
			//cout << "finding node in closed List" << endl;
			//now do the closedList -> it wont do it if isBetter is false
			for (tempIter = closedList.begin(); tempIter != closedList.end() && isBetter!=false; tempIter++) {
				//step though every element in the closedList
				if ((*tempIter)->x==successor[i].x && (*tempIter)->y==successor[i].y) {
					//successor is an closedList element
					if((*tempIter)->f <= successor[i].f) {
						//the OpenList element is better than the successor
						//as soon as a node is found that is better then stop using this successor
						//cout << "   There is a better version in the closedList" << endl;
						isBetter = false;
					} else {
						//the OpenList element is worse
						//remove all occurances that are worse than the successor
						//cout << "   Deleting worse version from closedList" << endl;
						isBetter = true;
						removeNode(*tempIter, tempIter);
						break;
					}
				}
			}

			//add successor to the open List -> set parent to be current node
			if (isBetter!=false) {
				//cout << "    ADDING successor["<< i <<"] to openList" << endl;
				newToList(successor[i].x, successor[i].y, currentNode, &openList);
			}
		   }
		}
	}

	//drawList(&closedList);
	//printOpenList();
	//printClosedList();

   }
}


void shortestRoute::calcRoute(int nodeX, int nodeY) {
//intitally finalNode is the node that got us to the goal
	cout << "calculating Route... stepping back from Node[" << nodeX << "," << nodeY << "]" << endl;
	//place x&y of the finalNode onto the front of the route's path - but convert it to globalUnits
	Vector nodePos = theMap->convMAPcoordtoGU(nodeX,nodeY);
	thePath->addPointFront(nodePos.x, nodePos.y);

	if (nodeX==startX && nodeY==startY) {
	//if it is then set the first point on the path to be the GU start and the last point to be the end point
		cout << "   route finding was a SUCCESS" << endl;
		thePath->pointList[0] = start;
		thePath->pointList[(thePath->pointList.size()-1)] = target;
		thePath->printPath();
		//the search is done, set it to success
		activityState = 1;
	} else {
		//if finalNode is not the same x&y as the start pos then call calcRoute with the finalNode's parent
		int nodeParentX = (*(findInList(nodeX, nodeY, &closedList)))->parentX;
		int nodeParentY = (*(findInList(nodeX, nodeY, &closedList)))->parentY;
		calcRoute(nodeParentX, nodeParentY);
	}
}

void shortestRoute::drawList(vector <AstarNode*> *whatList) {
	AstarList_Iter tempIter ;
	//Vector nodeCenter;
	//float maxValue = sqrt( (pCluster->pBot->pTheUniverse->globalMap.size.x*pCluster->pBot->pTheUniverse->globalMap.size.x) + (pCluster->pBot->pTheUniverse->globalMap.size.y*pCluster->pBot->pTheUniverse->globalMap.size.y) );
	for (tempIter=whatList->begin(); tempIter!=whatList->end(); tempIter++) {
		drawNode((*tempIter));
	}
}

void shortestRoute::drawNode(AstarNode* whatNode) {
	Vector nodeCenter = theMap->convMAPcoordtoGU(whatNode->x, whatNode->y);
	float GLradius = pCluster->pBot->pTheUniverse->globalMap.convToGL(theMap->dataResWidth/2);

	glColor4f( 1 , 0, 0, 0.5);
  	glPushMatrix();
		glTranslatef(nodeCenter.x,nodeCenter.y,0.1);
		glutSolidSphere(GLradius, 10, 2);
		//glBegin(GL_POLYGON);
			//pCluster->pBot->pTheUniverse->globalMap.GUToGLvertex(nodeCenter.x-(theMap->dataResWidth/2) , nodeCenter.y-(theMap->dataResHeight/2));
			//pCluster->pBot->pTheUniverse->globalMap.GUToGLvertex(nodeCenter.x+(theMap->dataResWidth/2) , nodeCenter.y-(theMap->dataResHeight/2));
			//pCluster->pBot->pTheUniverse->globalMap.GUToGLvertex(nodeCenter.x+(theMap->dataResWidth/2) , nodeCenter.y+(theMap->dataResHeight/2));
			//pCluster->pBot->pTheUniverse->globalMap.GUToGLvertex(nodeCenter.x-(theMap->dataResWidth/2) , nodeCenter.y+(theMap->dataResHeight/2));
		//glEnd();
	glPopMatrix();
}

//*********************************************************************************
