#include "map.h"

/*!var v2d globalMapDims how wide and high the world map is - assume its in metres... but could be in anything really ...lets call them GlobalUnits ---- when speeds etc are refered to as metres - they are actually in GUs*/
//v2d globalMapDims;

dataMap::dataMap()
{
	cout << "\n\n..........new Map.........." << endl;
	valueArray = NULL;

}

dataMap::~dataMap()
{
	//map destructor - called when map object removed from memory, clears data array
	if (valueArray!=NULL) {
		for (int x=0; x<dataWidth; x++)
			delete(valueArray[x]);
		delete (valueArray);
	}
}

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


void dataMap::assignMapFile(char* mapFileName) {

	Image mapFileImg;
	if(!mapFileImg.loadImage(mapFileName))
	{
	  printf("Unable to load map image!\n");
	  exit(EXIT_FAILURE);
	}

	dataWidth = mapFileImg.getX(); //number of data elements in x
   	dataHeight = mapFileImg.getY(); // number of data elements in y

	dataResWidth = (pTheUniverse->globalMap.size.x+1)/dataWidth;
	dataResHeight = (pTheUniverse->globalMap.size.y+1)/dataHeight;

	valueArray = new (float*)[dataWidth];
	for (int i=0; i<dataWidth; i++) {
		valueArray[i] = new (float)[dataHeight];
	}

	cout << "Adding Map["<< universeID << "] - '"<< mapFileName << "' w:" << dataWidth << " h:" << dataHeight << "  ResW:" << dataResWidth << "  ResH:" << dataResHeight << endl;
	float luminance, r, g, b;
	float cnt=0;
	int x = 0;
	for(int i=0; i<dataHeight; i++) {
     		for(int j=0; j<dataWidth; j++) {
			//calc luminance
			r = (float)mapFileImg.data[x] / 255;
			g = (float)mapFileImg.data[x+1] / 255;
			b = (float)mapFileImg.data[x+2] / 255;
			luminance = (r+g+b)/3;
			cout << (int)((cnt/(dataWidth*dataHeight))*100) << "% Loaded\r";
			//cout << " > adding[" << j << "," << i << "] : r:"<< r << " g:" << g << " b:" << b << " lum:" << luminance << endl;

			setValue(luminance, j, i);
			x += 3;
			cnt++;
		}
	}

}

void dataMap::drawMap() {
float tempValue;
Vector dataCenter;

	for (int i=0; i<dataWidth; i++) {
		for (int j = 0; j<dataHeight; j++) {
		   glPushMatrix();
			dataCenter = convMAPcoordtoGU(i, j);
			tempValue = getValue(i,j);
			//cout << "drawing Map[" << i << "," << j << "]("<< dataCenter.x << "," << dataCenter.y << "): " << tempValue << endl;
			glColor3f( tempValue , tempValue, tempValue);
			glBegin(GL_POLYGON);
				pTheUniverse->globalMap.GUToGLvertex(dataCenter.x-(dataResWidth/2) , dataCenter.y-(dataResHeight/2));
				pTheUniverse->globalMap.GUToGLvertex(dataCenter.x+(dataResWidth/2) , dataCenter.y-(dataResHeight/2));
				pTheUniverse->globalMap.GUToGLvertex(dataCenter.x+(dataResWidth/2) , dataCenter.y+(dataResHeight/2));
				pTheUniverse->globalMap.GUToGLvertex(dataCenter.x-(dataResWidth/2) , dataCenter.y+(dataResHeight/2));
			glEnd();
  		   glPopMatrix();
		}
	}
}

void dataMap::setValue( double value, int xCoord, int yCoord)
{
	valueArray[xCoord][yCoord] = value;
}

Vector dataMap::convMAPcoordtoGU (int xCoord, int yCoord)
{
//take in integer map coords and return its position on the global map
	Vector tempCoord;
	tempCoord.x = ((double)xCoord * dataResWidth) +(dataResWidth/2);
	tempCoord.y = ((double)yCoord * dataResHeight) +(dataResHeight/2);

	return tempCoord;
}


Vector dataMap::convGUtoMAPcoord (float GUxCoord, float GUyCoord)
{
//take in a GU coord and return its position on the map
	Vector tempCoord;
		tempCoord.x = (GUxCoord / dataResWidth);
		tempCoord.y = (GUyCoord / dataResHeight);

	return tempCoord;
	//this returns a double - so it is inbetween data elements - need data interpolation
}



Vector dataMap::nearestMAPcoord(float GUxCoord, float GUyCoord)
{
	Vector intCoord;
	Vector tempMapCoord = convGUtoMAPcoord (GUxCoord, GUyCoord);

	if((tempMapCoord.x-floor(tempMapCoord.x)) < 0.5 ) {
		//to the left
		intCoord.x = (int)floor(tempMapCoord.x);
	} else {
		//to the right
		intCoord.x = (int)ceil(tempMapCoord.x);
	}

	if((tempMapCoord.y-floor(tempMapCoord.y)) < 0.5 ) {
		//to the bottom
		intCoord.y = (int)floor(tempMapCoord.y);
	} else {
		//to the top
		intCoord.y = (int)ceil(tempMapCoord.y);
	}

	return intCoord;
}

// the following methods have not been properly implemented, but the code is still here...

/*
double map::getInterpolatedValue(Vector GUCoord)
{
//find the average value of a GU coord on a map
	Vector mapPos = convGUtoMAPcoord(GUCoord);

	std::cout << "mapXpos:"<<mapPos.x<< " mapYpos:" << mapPos.y << std::endl;

	v2d bottLeft;
		bottLeft.x = floor(mapPos.x);
		bottLeft.y = floor(mapPos.y);

	v2d diff;
		diff.x = mapPos.x - bottLeft.x;
		diff.y = mapPos.y - bottLeft.y;

	v2d mapSize;
		mapSize.x = (double)getDataWidth();
		mapSize.y = (double)getDataHeight();
	if (bottLeft.x >= (mapSize.x-1)) {
		bottLeft.x = mapSize.x-2;
		diff.x = 1;
	}

	if (bottLeft.y >= (mapSize.y-1)) {
		bottLeft.y = mapSize.y-2;
		diff.y = 1;
	}

	std::cout << "bottLeftx:"<<bottLeft.x<< " bottLefty:" << bottLeft.y << std::endl;
	std::cout << "xDiff:"<<diff.x<< " yDiff:" << diff.y << std::endl;

	double quadValues[4];
	quadValues[0] = valueArray[ (int)bottLeft.x ][ (int)bottLeft.y ];
	std::cout << "bL:"<< quadValues[0] << std::endl;
	quadValues[1] = valueArray[ (int)(bottLeft.x+1) ][ (int)bottLeft.y ];
	std::cout << "bR:"<< quadValues[1] << std::endl;
	quadValues[2] = valueArray[ (int)(bottLeft.x+1) ][ (int)(bottLeft.y+1) ];
	std::cout << "tR:"<< quadValues[2] << std::endl;
	quadValues[3] = valueArray[ (int)bottLeft.x ][ (int)(bottLeft.y+1) ];
	std::cout << "tL:"<< quadValues[3] << std::endl;

	double bottomX = quadValues[0]*(1-diff.x) +quadValues[1]*(diff.x);
	double topX = quadValues[3]*(1-diff.x) +quadValues[2]*(diff.x);

	return bottomX*(1-diff.y) + topX*(diff.y);
}
*/
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*
int map::convMAPcoordtoIndex(int xCoord, int yCoord)
{
//take in integer map coords and the map resolution and return the map index
	return (dataWidth*yCoord) + xCoord;
}
*/

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

/*
intPnt map::convIndexToMAPCoord (int index)
{
//take in a map index and return the coord on the map
	intPnt tempCoord;
	tempCoord.x = index%dataWidth;
	tempCoord.y = (int)(index/dataWidth);

	return tempCoord;
}

v2d map::convMAPindextoGU (int index)
{
//take in a map index and return its position on the global map

	intPnt tempMapCoord = convIndexToMAPCoord(index);

	return convMAPcoordtoGU(tempMapCoord.x, tempMapCoord.y);
}
*/
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*
int map::convGUtoMAPindex (Vector GUCoord)
{
//take in a GU coord and return its index on the map
	Vector tempCoord = (int)nearestMAPcoord(GUCoord);

	return convMAPcoordtoIndex(tempCoord.x, tempCoord.y);
}
*/
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>