
#include "universeclass.h"

using namespace std;
using namespace GraphicsLib;

// TODO PUT SOMEWHERE ELSE
Vector truncateVec(Vector vecA, float max) {
	if (vecA.length() > max) {
		Vector tempVec = vecA;
		tempVec.normalize();
		return (tempVec*max);
	} else return vecA;
}


universe::universe (float universeWidth_in, float universeHeight_in) {
	screen tempGlobalMap(universeWidth_in, universeHeight_in);
	globalMap = tempGlobalMap;

	botList.clear();
	botGrpList.clear();
	botTypeList.clear();
	mapDataList.clear();
	numBots = 0;
	numBotGrps = 0;
	numBotTypes = 0;
	numMaps = 0;

}

universe::~universe() {

	for (int i=0; i<numBots; i++) {
		removeBot(i);
	}

	for (int i=0; i<numBotGrps; i++) {
		delete &botGrpList[i];
	}

	for (int i=0; i<numBotTypes; i++) {
		delete &botTypeList[i];
	}

	for (int i=0; i<numMaps; i++) {
	//	delete &mapDataList[i];
	}
	delete &globalMap;
	delete &masterClock;
}

void universe::doAction() {
//what to do every step

	for (int i=0; i<numMaps; i++) {
		mapDataList[i].drawMap();
	}

	for (int i = 0; i<numBots; i++) {
		//cout << "\n\nBOT[" << i << "] - ";
		botList[i].doAction();
	}

}

bot* universe::addBot(botGrp* pWhatGrp, botType* pWhatType) {

     bot tempBot;
	botList.push_back(tempBot);

	botList[numBots].pTheUniverse = this;
	botList[numBots].universeID = numBots;

	cout << "BOT[" << numBots << "] CREATED addr:" << &botList[numBots] << "  grp addr:" << pWhatGrp <<  endl;
	botList[numBots].AIbrain = new AIcluster();
	botList[numBots].AIbrain->AIthoughtList.clear();//clear the brain of thoughts
	botList[numBots].AIbrain->clusterLvl = 0; //make it the root cluster
	botList[numBots].AIbrain->pBot = &botList[numBots]; //tell the brain what bot its working on

	botList[numBots].pBotGrp = pWhatGrp;
	botList[numBots].pBotType = pWhatType;

	pWhatGrp->addBot(&botList[numBots]); //add the new bot to the groups list of bots
	numBots++;

	updateBotPointers();
	return &botList[numBots-1];
}

void universe::removeBot(int whatID) {
	cout << "Removing Bot["<< whatID <<"]" << endl;
	if (botList[whatID].pBotGrp!= NULL) botList[whatID].pBotGrp->deleteBot(whatID);
	if(botList[whatID].AIbrain!=NULL) delete botList[whatID].AIbrain;
	botList.erase(botList.begin() + whatID);

	numBots--;
	for (int i=whatID; i<numBots; i++) {
		botList[i].universeID --;
	}
	updateBotPointers();
}


botGrp* universe::addGrp(Vector startPos_in, Colour grpColour_in) {

     botGrp tempGrp;
	botGrpList.push_back(tempGrp);
	botGrpList[numBotGrps].startPos = startPos_in;
	botGrpList[numBotGrps].pTheUniverse = this;
	botGrpList[numBotGrps].universeID = numBotGrps;
	botGrpList[numBotGrps].grpColour = grpColour_in;
	cout << "GROUP[" << numBotGrps << "] CREATED addr:" << &botGrpList[numBotGrps] << endl;

	numBotGrps++;
	updateGrpPointers();

	printGrpList();

	return &botGrpList[numBotGrps-1];
}


botType* universe::addType(float mass_in, float radius_in, float comfort_in, float maxspeed_in, float maxforce_in, float NHoodDist_in, float NHoodAngle_in) {

     botType tempType;
     	tempType.pTheUniverse = this;
     	tempType.universeID = numBotTypes;
     	tempType.mass = mass_in;
	tempType.radius = radius_in;
	tempType.comfort_Dist = comfort_in;
	tempType.max_speed = maxspeed_in;
	tempType.max_force = maxforce_in;
	tempType.NHoodRadius = NHoodDist_in;
	tempType.NHoodAngle = NHoodAngle_in;
	botTypeList.push_back(tempType);

	numBotTypes++;

	return &botTypeList[numBotTypes-1];

}

dataMap* universe::addMap(char* dataMapFile) {

     dataMap tempMap;

	tempMap.pTheUniverse = this;
     	tempMap.universeID = numMaps;

	mapDataList.push_back(tempMap);

	mapDataList[numMaps].assignMapFile(dataMapFile);

	numMaps++;

	return &mapDataList[numMaps-1];
}

void universe::updateBotPointers() {
// when a bot is added
//as vector arrays change pointers when new elements are added we need to reassign the addresses of all the lists of bots
	cout << "\n----------------------\nUPDATING bot pointers: " << endl;
	for (int i=0; i<numBots; i++) {
		//update the botGrp list of bots
		cout << "    BOT[" << i << "] grp[" << botList[i].pBotGrp->universeID << "] oldAddr:" << botList[i].pBotGrp->pBotList[botList[i].grpID];
		botList[i].pBotGrp->pBotList[botList[i].grpID] = &botList[i];
		cout << "  newAddr:" << botList[i].pBotGrp->pBotList[botList[i].grpID] << endl;
		botList[i].AIbrain->pBot = &botList[i]; //tell the brain what bot its working on
	}

}

void universe::updateGrpPointers() {
// when a group is added
	for (int i=0; i<numBotGrps; i++) {
		for (int j=0; j<botGrpList[i].numBots; j++) {
			// update the bot's grp pointer
			botGrpList[i].pBotList[j]->pBotGrp = &botGrpList[i];
		}
	}
}

void universe::printGrpList() {
	cout << "\n\n--------GROUP LIST--------" << endl;
	for (int i=0; i<numBotGrps; i++) {
		cout << "Grp[" << botGrpList[i].universeID << "] numBots:" << botGrpList[i].numBots << "  startPos:" << botGrpList[i].startPos << endl;

	}
	cout << "\n\n";
}
